"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.APIReferenceModel = void 0;
function _semanticsEntities() {
  const data = require("@teambit/semantics.entities.semantic-schema");
  _semanticsEntities = function () {
    return data;
  };
  return data;
}
function _componentId() {
  const data = require("@teambit/component-id");
  _componentId = function () {
    return data;
  };
  return data;
}
class APIReferenceModel {
  internalAPIKey(schema) {
    return this.generateInternalAPIKey(schema.location.filePath, schema.name);
  }
  generateInternalAPIKey(filePath, name = '') {
    return `${filePath}/${name}`;
  }
  taggedAPINodes = [];
  constructor(_api, _renderers) {
    this._api = _api;
    this.componentId = _api.componentId;
    this.apiNodes = this.mapToAPINode(_api, _renderers, this.componentId);
    this.apiByType = this.groupByType(this.apiNodes);
    this.apiByName = this.groupByName(this.apiNodes);
    this.taggedAPINodes = this.mapTaggedAPINode(_api, _renderers, this.componentId);
  }
  mapTaggedAPINode(api, renderers, componentId) {
    const {
      taggedModuleExports
    } = api;
    const defaultRenderers = renderers.filter(renderer => renderer.default);
    const nonDefaultRenderers = renderers.filter(renderer => !renderer.default);
    return taggedModuleExports.map(schemaNode => ({
      componentId,
      api: schemaNode,
      exported: true,
      renderer: nonDefaultRenderers.find(renderer => renderer.predicate(schemaNode)) || defaultRenderers.find(renderer => renderer.predicate(schemaNode))
    })).filter(schemaNode => schemaNode.renderer);
  }
  mapToAPINode(api, renderers, componentId) {
    const {
      internals
    } = api;
    const {
      exports: exportedSchemaNodes
    } = api.module;
    const exportedInternalKeySet = new Set(exportedSchemaNodes.map(schemaNode => this.internalAPIKey(schemaNode)));
    const internalSchemaNodes = internals.reduce((acc, next) => {
      return acc.concat([...next.internals]);
    }, new Array()).filter(schemaNode => !exportedInternalKeySet.has(this.internalAPIKey(schemaNode)));
    const defaultRenderers = renderers.filter(renderer => renderer.default);
    const nonDefaultRenderers = renderers.filter(renderer => !renderer.default);
    return exportedSchemaNodes.map(schemaNode => ({
      componentId,
      api: _semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode,
      alias: _semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.alias : undefined,
      exported: true,
      renderer: nonDefaultRenderers.find(renderer => renderer.predicate(_semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode)) || defaultRenderers.find(renderer => renderer.predicate(_semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode))
    })).concat(internalSchemaNodes.map(schemaNode => ({
      componentId,
      api: schemaNode,
      exported: false,
      renderer: nonDefaultRenderers.find(renderer => renderer.predicate(schemaNode)) || defaultRenderers.find(renderer => renderer.predicate(schemaNode))
    }))).filter(schemaNode => schemaNode.renderer);
  }
  getByType(type) {
    return this.apiByType.get(type) ?? [];
  }
  getByName(node) {
    if (!node.name) return undefined;
    return this.apiByName.get(node.name) ?? this.apiByName.get(this.internalAPIKey(node));
  }
  groupByType(apiNodes) {
    return apiNodes.reduce((accum, next) => {
      const existing = accum.get(next.renderer.nodeType) || [];
      accum.set(next.renderer.nodeType, existing.concat(next));
      return accum;
    }, new Map());
  }
  groupByName(apiNodes) {
    return apiNodes.reduce((accum, next) => {
      const name = next.alias || next.api.name;
      if (!name) return accum;
      const key = next.exported ? name : this.internalAPIKey(next.api);
      accum.set(key, next);
      return accum;
    }, new Map());
  }
  static from(result, renderers = []) {
    try {
      const apiSchema = _semanticsEntities().APISchema.fromObject(result.getHost.getSchema);
      return new APIReferenceModel(apiSchema, renderers);
    } catch {
      return new APIReferenceModel(_semanticsEntities().APISchema.empty(_componentId().ComponentID.fromObject(result.getHost.getSchema.componentId)), renderers);
    }
  }
}
exports.APIReferenceModel = APIReferenceModel;

//# sourceMappingURL=api-reference-model.js.map